@@ -6,7 +6,7 @@
  * @licstart  The following is the entire license notice for the
  * JavaScript code in this file.
  *
- * Copyright (c) 2005-2014, The Roundcube Dev Team
+ * Copyright (c) The Roundcube Dev Team
  *
  * The JavaScript code in this page is free software: you can
  * redistribute it and/or modify it under the terms of the GNU
@@ -58,49 +58,56 @@ function roundcube_browser()
   this.dom = document.getElementById ? true : false;
   this.dom2 = document.addEventListener && document.removeEventListener;
 
-  this.webkit = this.agent_lc.indexOf('applewebkit') > 0;
+  this.edge = this.agent_lc.indexOf(' edge/') > 0 || this.agent_lc.indexOf(' edg/') > 0; // "edg" in Chromium based Edge
+  this.webkit = !this.edge && this.agent_lc.indexOf('applewebkit') > 0;
   this.ie = (document.all && !window.opera) || (this.win && this.agent_lc.indexOf('trident/') > 0);
 
   if (window.opera) {
     this.opera = true; // Opera < 15
     this.vendver = opera.version();
   }
-  else if (!this.ie) {
+  else if (!this.ie && !this.edge) {
     this.chrome = this.agent_lc.indexOf('chrome') > 0;
     this.opera = this.webkit && this.agent.indexOf(' OPR/') > 0; // Opera >= 15
     this.safari = !this.chrome && !this.opera && (this.webkit || this.agent_lc.indexOf('safari') > 0);
     this.konq = this.agent_lc.indexOf('konqueror') > 0;
     this.mz = this.dom && !this.chrome && !this.safari && !this.konq && !this.opera && this.agent.indexOf('Mozilla') >= 0;
-    this.iphone = this.safari && (this.agent_lc.indexOf('iphone') > 0 || this.agent_lc.indexOf('ipod') > 0);
-    this.ipad = this.safari && this.agent_lc.indexOf('ipad') > 0;
+    this.iphone = this.safari && (this.agent_lc.indexOf('iphone') > 0 || this.agent_lc.indexOf('ipod') > 0 || this.platform == 'ipod' || this.platform == 'iphone');
+    this.ipad = this.safari && (this.agent_lc.indexOf('ipad') > 0 || this.platform == 'ipad');
   }
 
   if (!this.vendver) {
-    // common version strings
-    this.vendver = /(opera|opr|khtml|chrome|safari|applewebkit|msie)(\s|\/)([0-9\.]+)/.test(this.agent_lc) ? parseFloat(RegExp.$3) : 0;
+    if (this.ie)
+      pattern = /(msie|rv)(\s|:)([0-9\.]+)/;
+    else if (this.edge)
+      pattern = /(edge?)(\/)([0-9\.]+)/;
+    else if (this.opera)
+      pattern = /(opera|opr)(\/)([0-9\.]+)/;
+    else if (this.konq)
+      pattern = /(konqueror)(\/)([0-9\.]+)/;
+    else if (this.safari)
+      pattern = /(version)(\/)([0-9\.]+)/;
+    else if (this.chrome)
+      pattern = /(chrome)(\/)([0-9\.]+)/;
+    else if (this.mz)
+      pattern = /(firefox)(\/)([0-9\.]+)/;
+    else
+      pattern = /(khtml|safari|applewebkit|rv)(\s|\/|:)([0-9\.]+)/;
 
-    // any other (Mozilla, Camino, IE>=11)
-    if (!this.vendver)
-      this.vendver = /rv:([0-9\.]+)/.test(this.agent) ? parseFloat(RegExp.$1) : 0;
+    // common version strings
+    this.vendver = pattern.test(this.agent_lc) ? parseFloat(RegExp.$3) : 0;
   }
 
   // get real language out of safari's user agent
   if (this.safari && (/;\s+([a-z]{2})-[a-z]{2}\)/.test(this.agent_lc)))
     this.lang = RegExp.$1;
 
-  this.tablet = /ipad|android|xoom|sch-i800|playbook|tablet|kindle/i.test(this.agent_lc);
   this.mobile = /iphone|ipod|blackberry|iemobile|opera mini|opera mobi|mobile/i.test(this.agent_lc);
+  this.tablet = !this.mobile && /ipad|android|xoom|sch-i800|playbook|tablet|kindle/i.test(this.agent_lc);
   this.touch = this.mobile || this.tablet;
+  this.pointer = typeof window.PointerEvent == "function";
   this.cookies = n.cookieEnabled;
 
-  // test for XMLHTTP support
-  this.xmlhttp_test = function()
-  {
-    var activeX_test = new Function("try{var o=new ActiveXObject('Microsoft.XMLHTTP');return true;}catch(err){return false;}");
-    this.xmlhttp = window.XMLHttpRequest || (('ActiveXObject' in window) && activeX_test());
-    return this.xmlhttp;
-  };
-
   // set class names to html tag according to the current user agent detection
   // this allows browser-specific css selectors like "html.chrome .someclass"
   this.set_html_class = function()
@@ -108,7 +115,11 @@ function roundcube_browser()
     var classname = ' js';
 
     if (this.ie)
-      classname += ' ie ie'+parseInt(this.vendver);
+      classname += ' ms ie ie'+parseInt(this.vendver);
+    else if (this.edge && this.vendver > 74)
+      classname += ' chrome';
+    else if (this.edge)
+      classname += ' ms edge';
     else if (this.opera)
       classname += ' opera';
     else if (this.konq)
@@ -147,7 +158,7 @@ var rcube_event = {
 get_target: function(e)
 {
   e = e || window.event;
-  return e && e.target ? e.target : e.srcElement;
+  return e && e.target ? e.target : e.srcElement || document;
 },
 
 /**
@@ -275,14 +286,21 @@ cancel: function(evt)
 },
 
 /**
- * Determine whether the given event was trigered from keyboard
+ * Determine whether the given event was triggered from keyboard
  */
 is_keyboard: function(e)
 {
-  return e && (
-      (e.type && String(e.type).match(/^key/)) // DOM3-compatible
-      || (!e.pageX && (e.pageY || 0) <= 0 && !e.clientX && (e.clientY || 0) <= 0) // others
-    );
+  if (!e)
+    return false;
+
+  // DOM3-compatible
+  // An event invoked by pressing Enter on a link will produce a 'click' event,
+  // so we have to extend the check, e.g. with use of e.clientX.
+  if (e.type)
+    return !!e.type.match(/^key/) || (e.type == 'click' && !e.clientX);
+
+  // Old browsers
+  return !e.pageX && (e.pageY || 0) <= 0 && !e.clientX && (e.clientY || 0) <= 0;
 },
 
 /**
@@ -353,7 +371,10 @@ removeEventListener: function(evt, func, obj)
  */
 triggerEvent: function(evt, e)
 {
-  var ret, h;
+  var ret, h,
+    reset_fn = function(o) {
+      try { if (o && o.event) delete o.event; } catch(err) { };
+    };
 
   if (e === undefined)
     e = this;
@@ -377,26 +398,11 @@ triggerEvent: function(evt, e)
           break;
       }
     }
-    if (ret && ret.event) {
-      try {
-        delete ret.event;
-      } catch (err) {
-        // IE6-7 doesn't support deleting HTMLFormElement attributes (#1488017)
-        $(ret).removeAttr('event');
-      }
-    }
+    reset_fn(ret);
   }
 
   delete this._event_exec[evt];
-
-  if (e.event) {
-    try {
-      delete e.event;
-    } catch (err) {
-      // IE6-7 doesn't support deleting HTMLFormElement attributes (#1488017)
-      $(e).removeAttr('event');
-    }
-  }
+  reset_fn(e);
 
   return ret;
 }
@@ -407,7 +413,7 @@ triggerEvent: function(evt, e)
 // check if input is a valid email address
 // By Cal Henderson <cal@iamcal.com>
 // http://code.iamcal.com/php/rfc822/
-function rcube_check_email(input, inline, count)
+function rcube_check_email(input, inline, count, strict)
 {
   if (!input)
     return count ? 0 : false;
@@ -419,15 +425,15 @@ function rcube_check_email(input, inline, count)
       atom = '[^\\x00-\\x20\\x22\\x28\\x29\\x2c\\x2e\\x3a-\\x3c\\x3e\\x40\\x5b-\\x5d\\x7f-\\xff]+',
       quoted_pair = '\\x5c[\\x00-\\x7f]',
       quoted_string = '\\x22('+qtext+'|'+quoted_pair+')*\\x22',
-      ipv4 = '\\[(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])(\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])){3}\\]',
+      ipv4 = '\\[(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])(\\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])){3}\\]',
       ipv6 = '\\[IPv6:[0-9a-f:.]+\\]',
       ip_addr = '(' + ipv4 + ')|(' + ipv6 + ')',
       // Use simplified domain matching, because we need to allow Unicode characters here
       // So, e-mail address should be validated also on server side after idn_to_ascii() use
       //domain_literal = '\\x5b('+dtext+'|'+quoted_pair+')*\\x5d',
       //sub_domain = '('+atom+'|'+domain_literal+')',
-      // allow punycode/unicode top-level domain
-      domain = '(('+ip_addr+')|(([^@\\x2e]+\\x2e)+([^\\x00-\\x40\\x5b-\\x60\\x7b-\\x7f]{2,}|xn--[a-z0-9]{2,})))',
+      // allow punycode/unicode top-level domain, allow extended domains (#5588)
+      domain = '(('+ip_addr+')|(([^@\\x2e]+\\x2e)+([^\\x00-\\x2f\\x3a-\\x40\\x5b-\\x60\\x7b-\\x7f]{2,}|xn--[a-z0-9]{2,})))',
       // ICANN e-mail test (http://idn.icann.org/E-mail_test)
       icann_domains = [
         '\\u0645\\u062b\\u0627\\u0644\\x2e\\u0625\\u062e\\u062a\\u0628\\u0627\\u0631',
@@ -437,13 +443,13 @@ function rcube_check_email(input, inline, count)
         '\\u0909\\u0926\\u093e\\u0939\\u0930\\u0923\\x2e\\u092a\\u0930\\u0940\\u0915\\u094d\\u0937\\u093e',
         '\\u4f8b\\u3048\\x2e\\u30c6\\u30b9\\u30c8',
         '\\uc2e4\\ub840\\x2e\\ud14c\\uc2a4\\ud2b8',
-        '\\u0645\\u062b\\u0627\\u0644\\x2e\\u0622\\u0632\\u0645\\u0627\\u06cc\\u0634\u06cc',
+        '\\u0645\\u062b\\u0627\\u0644\\x2e\\u0622\\u0632\\u0645\\u0627\\u06cc\\u0634\\u06cc',
         '\\u043f\\u0440\\u0438\\u043c\\u0435\\u0440\\x2e\\u0438\\u0441\\u043f\\u044b\\u0442\\u0430\\u043d\\u0438\\u0435',
         '\\u0b89\\u0ba4\\u0bbe\\u0bb0\\u0ba3\\u0bae\\u0bcd\\x2e\\u0baa\\u0bb0\\u0bbf\\u0b9f\\u0bcd\\u0b9a\\u0bc8',
         '\\u05d1\\u05f2\\u05b7\\u05e9\\u05e4\\u05bc\\u05d9\\u05dc\\x2e\\u05d8\\u05e2\\u05e1\\u05d8'
       ],
       icann_addr = 'mailtest\\x40('+icann_domains.join('|')+')',
-      word = '('+atom+'|'+quoted_string+')',
+      word = strict ? '('+atom+'|'+quoted_string+')' : '[^\\u0000-\\u0020\\u002e\\u00a0\\u0040\\u007f\\u2028\\u2029]+',
       delim = '[,;\\s\\n]',
       local_part = word+'(\\x2e'+word+')*',
       addr_spec = '(('+local_part+'\\x40'+domain+')|('+icann_addr+'))',
@@ -542,7 +548,7 @@ function setCookie(name, value, expires, path, domain, secure)
       (expires ? "; expires=" + expires.toGMTString() : "") +
       (path ? "; path=" + path : "") +
       (domain ? "; domain=" + domain : "") +
-      (secure ? "; secure" : "");
+      (secure ? "; secure" : "") + '; SameSite=Lax';
 
   document.cookie = curCookie;
 };
@@ -610,6 +616,18 @@ if (!String.prototype.startsWith) {
   };
 }
 
+if (!String.prototype.endsWith) {
+  String.prototype.endsWith = function(searchString, position) {
+    var subjectString = this.toString();
+    if (typeof position !== 'number' || !isFinite(position) || Math.floor(position) !== position || position > subjectString.length) {
+      position = subjectString.length;
+    }
+    position -= searchString.length;
+    var lastIndex = subjectString.lastIndexOf(searchString, position);
+    return lastIndex !== -1 && lastIndex === position;
+  };
+}
+
 // array utility function
 jQuery.last = function(arr) {
   return arr && arr.length ? arr[arr.length-1] : undefined;
